/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.cache.ehcache.internal.configurator;

import com.liferay.portal.cache.configuration.PortalCacheManagerConfiguration;
import com.liferay.portal.kernel.util.PropsKeys;
import com.liferay.portal.kernel.util.StringPool;

import net.sf.ehcache.config.Configuration;
import net.sf.ehcache.config.FactoryConfiguration;

import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;

/**
 * @author Tina Tian
 */
@Component(
	enabled = false, immediate = true,
	service = MultiVMEhcachePortalCacheManagerConfigurator.class
)
public class RMIMultiVMEhcachePortalCacheManagerConfigurator
	extends MultiVMEhcachePortalCacheManagerConfigurator {

	@Activate
	@Override
	protected void activate() {
		super.activate();

		if (!clusterEnabled) {
			return;
		}

		_peerListenerFactoryClass = props.get(
			PropsKeys.EHCACHE_RMI_PEER_LISTENER_FACTORY_CLASS);
		_peerListenerFactoryPropertiesString = getPortalPropertiesString(
			PropsKeys.EHCACHE_RMI_PEER_LISTENER_FACTORY_PROPERTIES);
		_peerProviderFactoryClass = props.get(
			PropsKeys.EHCACHE_RMI_PEER_PROVIDER_FACTORY_CLASS);
		_peerProviderFactoryPropertiesString = getPortalPropertiesString(
			PropsKeys.EHCACHE_RMI_PEER_PROVIDER_FACTORY_PROPERTIES);
	}

	@Override
	@SuppressWarnings("rawtypes")
	protected void manageConfiguration(
		Configuration configuration,
		PortalCacheManagerConfiguration portalCacheManagerConfiguration) {

		if (!clusterEnabled) {
			return;
		}

		super.manageConfiguration(
			configuration, portalCacheManagerConfiguration);

		FactoryConfiguration peerProviderFactoryConfiguration =
			new FactoryConfiguration();

		peerProviderFactoryConfiguration.setClass(_peerProviderFactoryClass);
		peerProviderFactoryConfiguration.setProperties(
			_peerProviderFactoryPropertiesString);
		peerProviderFactoryConfiguration.setPropertySeparator(StringPool.COMMA);

		configuration.addCacheManagerPeerProviderFactory(
			peerProviderFactoryConfiguration);

		FactoryConfiguration peerListenerFacotryConfiguration =
			new FactoryConfiguration();

		peerListenerFacotryConfiguration.setClass(_peerListenerFactoryClass);
		peerListenerFacotryConfiguration.setProperties(
			_peerListenerFactoryPropertiesString);
		peerListenerFacotryConfiguration.setPropertySeparator(StringPool.COMMA);

		configuration.addCacheManagerPeerListenerFactory(
			peerListenerFacotryConfiguration);
	}

	private String _peerListenerFactoryClass;
	private String _peerListenerFactoryPropertiesString;
	private String _peerProviderFactoryClass;
	private String _peerProviderFactoryPropertiesString;

}